// naumann@stce.rwth-aachen.de

#include <Eigen/Dense>
using VT=Eigen::VectorX<double>;

#include <cassert>
#include <cmath>
#include <iostream>
using namespace std;

double f(double p, double x) {
  assert(x>=0);
  return p*x;
}

double e(double p, VT xobs, VT yobs) {
  assert(xobs.size()==yobs.size());
  double e_=0;
  int i=0;
  while (i<xobs.size()) {
    e_=e_+pow(f(p,xobs(i))-yobs(i),2);
    i=i+1;
  }
  return e_;
}

VT minimize(double p, VT xobs,VT yobs) {
  assert(xobs.size()==yobs.size());
  VT result(2);
  result(0)=p; 
  result(1)=e(p,xobs,yobs);
  p=p-1;
  int i=0;
  while (i<200) {
    p=p+0.01;
    double e_=e(p,xobs,yobs);
    if (e_<result(1)) {
      result(0)=p;
      result(1)=e_;
    }
    i=i+1;
  }
  return result;
}

int main() {
  double p=0;
  int nobs=5;
  VT xobs=VT::Random(nobs); 
  xobs=xobs.cwiseProduct(xobs); 
  VT yobs=VT::Random(nobs); 
  VT m=minimize(p,xobs,yobs);
  cout << "p=" << m(0) << "; error=" << m(1) << std::endl;
  return 0;
}
