// naumann@stce.rwth-aachen.de

#include <Eigen/Dense>

template<typename T>
using VT=Eigen::VectorX<T>;

#include <cassert>
#include <fstream>
#include <iostream>
using namespace std;

template<typename T>
void fit(VT<T> x, T &p, VT<T> y) { 
  assert(x.size()==y.size());
  VT<T> v=x; v(0)=v(0)-x(0)/fabs(x(0))*x.norm();
  VT<T> w=2*v/v.dot(v);
  x=x-w*v.dot(x);
  y=y-w*v.dot(y); 
  p=y(0)/x(0);
}

template<typename T>
void plot(VT<T> x, VT<T> y) { 
  assert(x.size()==y.size());
  ofstream o("data");
  for (int i=0;i<x.size();++i) {
    o << x(i) << ' ' << y(i) << endl;
  }
}

int main() {
  int m=5;
  using T=double;
  T p;
  VT<T> x=VT<T>::Random(m), y=VT<T>::Random(m);
  plot(x,y);
  y=y+x;
  fit(x,p,y);
  cout << "p=" << p << endl;
  cout << "||x*p-(y+x)||=" << (x*p-(y+x)).norm() << endl;
  cout << "||x^T*x*p-x^T*(y+x)||=" << (x.transpose()*x*p-x.transpose()*(y+x)).norm() << endl;
  return 0;
}
