// naumann@stce.rwth-aachen.de

#include "dco.hpp"

#include <cmath>
using namespace std;

template<typename T>
T f(T p, T x) {
  return exp(sin(p*pow(x,2)));
}

template<typename T>
T df_dx_sym(T p, T x) {
  return exp(sin(p*pow(x,2)))*cos(p*pow(x,2))*2*p*x;
}

template<typename T>
T df_dx_cfd(T p, T x) {
  T h=1e-7;
  if (x!=0) h=h*fabs(x);
  return (f(p,x+h)-f(p,x-h))/(h+h);
}

template<typename T>
T df_dx_tad(T p, T x) {
  using TT=dco::gt1s<T>::type;
  TT xTT, yTT;
  xTT=x;
  dco::derivative(xTT)=1;
  yTT=f<TT>(p,xTT);
  return dco::derivative(yTT);
}

template<typename T>
T df_dx(T p, T x) {
  return df_dx_tad(p,x);
  return df_dx_cfd(p,x);
  return df_dx_sym(p,x);
}

template<typename T>
T ddf_dx_dx(T p, T x) {
  using TT=dco::gt1s<T>::type;
  TT xTT, dfTT;
  xTT=x;
  dco::derivative(xTT)=1;
  dfTT=df_dx<TT>(p,xTT);
  return dco::derivative(dfTT);
}

template<typename T>
T dddf_dx_dx_dx(T p, T x) {
  using TT=dco::gt1s<T>::type;
  TT xTT, ddfTT;
  xTT=x;
  dco::derivative(xTT)=1;
  ddfTT=ddf_dx_dx<TT>(p,xTT);
  return dco::derivative(ddfTT);
}

template<typename T>
T newton(T p, T x) {
  T dfdx=df_dx(p,x);
  do {
    T ddfdxx=ddf_dx_dx(p,x);
    T dddfdxxx=dddf_dx_dx_dx(p,x);
    if (fabs(dfdx*dddfdxxx/pow(ddfdxx,2))<=1) {
      cout << '+';
    } else {
      cout << '-';
    }
    x=x-dfdx/ddfdxx;
    dfdx=df_dx(p,x);
  } while (fabs(dfdx)>1e-7);
  cout << endl;
  return x;
}

int main() {
  using T=double;
  T p=1, x=1;
  cout << "y=" << f(p,x) << endl;
  x=newton(p,x);
  cout << "x=" << x << endl;
  cout << "y=" << f(p,x) << endl;
  cout << "dfdx=" << df_dx(p,x) << endl;
  cout << "ddfdxx=" << ddf_dx_dx(p,x) << endl;
  return 0;
}
