// naumann@stce.rwth-aachen.de

#include <Eigen/Dense>
#include "dco.hpp"

#include <cmath>
using namespace std;

template<typename T>
T f(T p, T x) {
  return exp(sin(p*pow(x,2)));
}

template<typename T>
T df_dx_sym(T p, T x) {
  return exp(sin(p*pow(x,2)))*cos(p*pow(x,2))*2*p*x;
}

template<typename T>
T df_dx_cfd(T p, T x) {
  T h=1e-7;
  if (x!=0) h=h*fabs(x);
  return (f(p,x+h)-f(p,x-h))/(h+h);
}

template<typename T>
T df_dx_tad(T p, T x) {
  using TT=dco::gt1s<T>::type;
  TT xTT, yTT;
  xTT=x;
  dco::derivative(xTT)=1;
  yTT=f<TT>(p,xTT);
  return dco::derivative(yTT);
}

template<typename T>
T df_dx(T p, T x) {
  return df_dx_sym(p,x);
  return df_dx_cfd(p,x);
  return df_dx_tad(p,x);
}

template<typename T>
T ddf_dx_dx(T p, T x) {
  using TT=dco::gt1s<T>::type;
  TT xTT, dfTT;
  xTT=x;
  dco::derivative(xTT)=1;
  dfTT=df_dx_tad<TT>(p,xTT);
  return dco::derivative(dfTT);
}

template<typename T>
T gradient_descent(T p, T x) {
  T dfdx=df_dx(p,x); 
  do {
    T alpha=1;
    T y=f(p,x); 
    while (f(p,x-alpha*dfdx)>y) { 
      alpha=alpha/2; 
    }
    x=x-alpha*dfdx; 
    dfdx=df_dx(p,x); 
    cout << '+';
  } while (fabs(dfdx)>1e-7);
  cout << endl;
  return x;
}

int main() {
  using T=double;
  T p=1, x=1;
  cout << "y=" << f(p,x) << endl;
  x=gradient_descent(p,x);
  cout << "x=" << x << endl;
  cout << "y=" << f(p,x) << endl;
  cout << "dfdx=" << df_dx(p,x) << endl;
  cout << "ddfdxx=" << ddf_dx_dx(p,x) << endl;
  return 0;
}
