// naumann@stce.rwth-aachen.de

#pragma once

#include "config.h"

#include "dco.hpp"

template<typename T> 
T f(T p, T x);

/// residual of model predictions relative to observations
template<typename T>
VT<T> r(T p, VT<T> xobs, VT<T> yobs) {
  assert(xobs.size()==nobs);
  assert(xobs.size()==yobs.size());
  VT<T> r_(nobs);
  int i=0;
  while (i<nobs) {
    r_(i)=f(p,xobs(i))-yobs(i);
    i=i+1;
  }
  return r_;
}

/// derivative of residual wrt. model parameter 
template<typename T> 
VT<T> dr_dp(T p, VT<T> xobs) { 
  assert(xobs.size()==nobs);
  VT<T> drdp(nobs);
  using TT=dco::gt1s<T>::type;
  TT pTT, yTT;
  int i=0;
  while (i<nobs) {
    pTT=p;
    dco::derivative(pTT)=1;
    yTT=f<TT>(pTT,xobs(i));
    drdp(i)=dco::derivative(yTT);
    i=i+1;
  }
  return drdp;
}
